@@ -17,18 +17,20 @@ import sphinx_rtd_theme
 
 from recommonmark.parser import CommonMarkParser
 
+from tom_base import __version__
+
 sys.path.insert(0, os.path.abspath('..'))
 
 
 # -- Project information -----------------------------------------------------
 
 project = 'TOM Toolkit'
-copyright = '2020, David Collom and Lindy Lindstrom'
-author = 'David Collom, Lindy Lindstrom, Austin Riba'
+copyright = '2021-4, Las Cumbres Observatory'
+author = 'Joey Chatelain, David Collom, Lindy Lindstrom, Austin Riba'
 
 # The full version, including alpha/beta/rc tags
 # This has to mirror the setup.py version for PDF generation
-release = '1.1.0'
+release = __version__
 
 # -- Django Configuration -------------------------------------------------
 
@@ -47,8 +49,11 @@ django.setup()
 # ones.
 extensions = [
     'sphinx.ext.autodoc',
+    'sphinx.ext.autosectionlabel',
     'sphinx.ext.autosummary',
-    'sphinx.ext.intersphinx'
+    'sphinx.ext.intersphinx',
+    'sphinx_design',
+    'sphinx_copybutton',
 ]
 
 # Add any paths that contain templates here, relative to this directory.
@@ -101,6 +106,8 @@ html_theme_options = {
     'github_button': 'false',
 }
 
+pygments_style = 'sphinx'
+
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
