@@ -2,7 +2,6 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from datetime import datetime
 from importlib import import_module
-import json
 
 from django import forms
 from django.conf import settings
@@ -17,13 +16,13 @@ from tom_targets.models import Target
 
 
 DEFAULT_ALERT_CLASSES = [
-    'tom_alerts.brokers.mars.MARSBroker',
     'tom_alerts.brokers.lasair.LasairBroker',
     'tom_alerts.brokers.scout.ScoutBroker',
     'tom_alerts.brokers.alerce.ALeRCEBroker',
     'tom_alerts.brokers.antares.ANTARESBroker',
     'tom_alerts.brokers.gaia.GaiaBroker',
-    'tom_alerts.brokers.scimma.SCIMMABroker',
+    'tom_alerts.brokers.fink.FinkBroker',  # the stub for the plugin
+    'tom_alerts.brokers.hermes.HermesBroker',  # the stub for the plugin
 ]
 
 
@@ -124,15 +123,6 @@ class GenericQueryForm(forms.Form):
         self.helper.add_input(Submit('submit', 'Submit'))
         self.common_layout = Layout('query_name', 'broker')
 
-    def serialize_parameters(self):
-        """
-        Returns a JSON-serialized representation of the form data.
-
-        :returns: JSON-ified form parameters
-        :rtype: str
-        """
-        return json.dumps(self.cleaned_data)
-
     def save(self, query_id=None):
         """
         Saves the form data in the database as a ``BrokerQuery``.
@@ -146,7 +136,7 @@ class GenericQueryForm(forms.Form):
             query = BrokerQuery()
         query.name = self.cleaned_data['query_name']
         query.broker = self.cleaned_data['broker']
-        query.parameters = self.serialize_parameters()
+        query.parameters = self.cleaned_data
         query.save()
         return query
 
@@ -188,27 +178,28 @@ class GenericBroker(ABC):
     https://github.com/TOMToolkit/tom_base/blob/main/tom_alerts/brokers/mars.py
     """
     alert_submission_form = GenericUpstreamSubmissionForm
+    score_description = "The meaning of this field changes between brokers, please consult this broker's documentation."
 
     @abstractmethod
-    def fetch_alerts(self, parameters):
+    def fetch_alerts(self, parameters: dict):
         """
         This method takes in the query parameters needed to filter
         alerts for a broker and makes the GET query to the broker
         endpoint.
 
         :param parameters: JSON string of query parameters
-        :type parameters: str
+        :type parameters: dict
         """
 
-    def fetch_alert(self, id):
+    def fetch_alert(self, alert_id):
         """
         This method takes an alert id and retrieves the specific
         alert data from the given broker.
 
-        :param id: Broker-specific id corresponding with the desired alert
-        :type id: str
+        :param alert_id: Broker-specific id corresponding with the desired alert
+        :type alert_id: str
         """
-        pass
+        return None
 
     def process_reduced_data(self, target, alert=None):
         """
@@ -221,7 +212,6 @@ class GenericBroker(ABC):
         :param alert: alert data from a particular ``BrokerQuery``
         :type alert: str
         """
-        pass
 
     def to_target(self, alert):
         """
@@ -230,7 +220,7 @@ class GenericBroker(ABC):
         :param alert: alert data from a particular ``BrokerQuery``
         :type alert: str
         """
-        pass
+        return None
 
     def submit_upstream_alert(self, target=None, observation_record=None, **kwargs):
         """
@@ -246,18 +236,17 @@ class GenericBroker(ABC):
         :returns: True or False depending on success of message submission
         :rtype: bool
         """
-        pass
 
     @abstractmethod
     def to_generic_alert(self, alert):
         """
         This method creates a ``GenericAlert`` object from the broker-specific
-        alert data for use outside of the implementation of the ``GenericBroker``.
+        alert data for use outside the implementation of the ``GenericBroker``.
 
         :param alert: alert data from a particular ``BrokerQuery``
         :type alert: str
         """
-        pass
+        return None
 
     def fetch_and_save_all(self, parameters):
         """
@@ -277,3 +266,10 @@ class GenericBroker(ABC):
             targets.append(target)
 
         return targets
+
+    def get_broker_context_data(self, *args, **kwargs):
+        """Override this method in the subclass to allow the Broker to add additional context
+        data to the View. This method will be called by views.RunQueryView.get_context_data()
+        and the returned dictionary will be added to the View's context.
+        """
+        return {}
