@@ -1,52 +1,101 @@
-from tom_alerts.alerts import GenericQueryForm, GenericAlert, GenericBroker
-from tom_alerts.models import BrokerQuery
-from tom_targets.models import Target
-from tom_dataproducts.models import ReducedDatum
 from dateutil.parser import parse
-from django import forms
+import json
+import re
+import requests
+from requests.exceptions import HTTPError
+
 from astropy.coordinates import SkyCoord
 from astropy.time import Time, TimezoneInfo
 import astropy.units as u
-import requests
-from requests.exceptions import HTTPError
-import json
-from os import path
+from bs4 import BeautifulSoup
+from crispy_forms.layout import Fieldset, HTML, Layout
+from django import forms
 
-BROKER_URL = 'http://gsaweb.ast.cam.ac.uk/alerts/alertsindex'
+from tom_alerts.alerts import GenericAlert, GenericBroker, GenericQueryForm
+from tom_dataproducts.models import ReducedDatum
+
+BASE_BROKER_URL = 'http://gsaweb.ast.cam.ac.uk'
 
 
 class GaiaQueryForm(GenericQueryForm):
-    target_name = forms.CharField(required=False)
+    target_name = forms.CharField(required=False, label='Target Name')
     cone = forms.CharField(
         required=False,
         label='Cone Search',
         help_text='RA,Dec,radius in degrees'
     )
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.helper.layout = Layout(
+            HTML('''
+                <p>
+                Please see the <a href="http://gsaweb.ast.cam.ac.uk/alerts/tableinfo" target="_blank">Gaia homepage</a>
+                for a detailed description of this broker.
+            '''),
+            self.common_layout,
+            Fieldset(
+                None,
+                'target_name',
+                'cone'
+            )
+        )
+
+    def clean_cone(self):
+        cone = self.cleaned_data['cone']
+        if cone:
+            cone_params = cone.split(',')
+            if len(cone_params) != 3:
+                raise forms.ValidationError('Cone search parameters must be in the format \'RA,Dec,Radius\'.')
+        return cone
+
     def clean(self):
-        if len(self.cleaned_data['target_name']) == 0 and \
-                        len(self.cleaned_data['cone']) == 0:
-            raise forms.ValidationError(
-                "Please enter either a target name or cone search parameters"
-                )
+        super().clean()
+        if not (self.cleaned_data.get('target_name') or self.cleaned_data.get('cone')):
+            raise forms.ValidationError('Please enter either a target name or cone search parameters.')
+        elif self.cleaned_data.get('target_name') and self.cleaned_data.get('cone'):
+            raise forms.ValidationError('Please only enter one of target name or cone search parameters.')
 
 
 class GaiaBroker(GenericBroker):
+    """
+    The ``GaiaBroker`` is the interface to the Gaia alert broker.
+
+    To include the ``GaiaBroker`` in your TOM, add the broker module location to your `TOM_ALERT_CLASSES` list in
+    your ``settings.py``:
+
+    .. code-block:: python
+
+        TOM_ALERT_CLASSES = [
+            'tom_alerts.brokers.gaia.GaiaBroker',
+            ...
+        ]
+
+    For information regarding the Gaia Science Alerts
+    Project, please see http://gsaweb.ast.cam.ac.uk/alerts/about.
+    """
+
     name = 'Gaia'
     form = GaiaQueryForm
 
     def fetch_alerts(self, parameters):
         """Must return an iterator"""
-        response = requests.get(BROKER_URL)
+        broker_feedback = ''
+        response = requests.get(f'{BASE_BROKER_URL}/alerts/alertsindex')
         response.raise_for_status()
 
-        html_data = response.text.split('\n')
-        for line in html_data:
-            if 'var alerts' in line:
-                alerts_data = line.replace('var alerts = ', '')
-                alerts_data = alerts_data.replace('\n', '').replace(';', '')
+        soup = BeautifulSoup(response.content, 'html.parser')
+        script_tags = soup.find_all('script')
+        alerts = None
 
-        alert_list = json.loads(alerts_data)
+        alerts_pattern = re.compile(r'var alerts = \[(.*?)];')
+        for script in script_tags:
+            m = alerts_pattern.match(str(script.string).strip())
+            if m is not None:
+                alerts = '['+m.group(1)+']'
+                break
+
+        alert_list = json.loads(alerts)
 
         if parameters['cone'] is not None and len(parameters['cone']) > 0:
             cone_params = parameters['cone'].split(',')
@@ -58,8 +107,7 @@ class GaiaBroker(GenericBroker):
                                                  frame="icrs", unit="deg")
 
         filtered_alerts = []
-        if parameters['target_name'] is not None and \
-                len(parameters['target_name']) > 0:
+        if parameters.get('target_name'):
             for alert in alert_list:
                 if parameters['target_name'] in alert['name']:
                     filtered_alerts.append(alert)
@@ -74,7 +122,7 @@ class GaiaBroker(GenericBroker):
         else:
             filtered_alerts = alert_list
 
-        return iter(filtered_alerts)
+        return iter(filtered_alerts), broker_feedback
 
     def fetch_alert(self, target_name):
 
@@ -87,7 +135,8 @@ class GaiaBroker(GenericBroker):
 
     def to_generic_alert(self, alert):
         timestamp = parse(alert['obstime'])
-        url = BROKER_URL.replace('/alerts/alertsindex', alert['per_alert']['link'])
+        alert_link = alert.get('per_alert', {})['link']
+        url = f'{BASE_BROKER_URL}/{alert_link}'
 
         return GenericAlert(
             timestamp=timestamp,
@@ -102,8 +151,6 @@ class GaiaBroker(GenericBroker):
 
     def process_reduced_data(self, target, alert=None):
 
-        base_url = BROKER_URL.replace('/alertsindex', '/alert')
-
         if not alert:
             try:
                 alert = self.fetch_alert(target.name)
@@ -111,13 +158,14 @@ class GaiaBroker(GenericBroker):
             except HTTPError:
                 raise Exception('Unable to retrieve alert information from broker')
 
-        alert_url = BROKER_URL.replace('/alerts/alertsindex',
-                                       alert['per_alert']['link'])
-
-        if alert:
-            lc_url = path.join(base_url, alert['name'], 'lightcurve.csv')
+        if alert is not None:
+            alert_name = alert['name']
+            alert_link = alert.get('per_alert', {})['link']
+            lc_url = f'{BASE_BROKER_URL}/alerts/alert/{alert_name}/lightcurve.csv'
+            alert_url = f'{BASE_BROKER_URL}/{alert_link}'
         elif target:
-            lc_url = path.join(base_url, target.name, 'lightcurve.csv')
+            lc_url = f'{BASE_BROKER_URL}/{target.name}/lightcurve.csv'
+            alert_url = f'{BASE_BROKER_URL}/alerts/alert/{target.name}/'
         else:
             return
 
@@ -138,13 +186,12 @@ class GaiaBroker(GenericBroker):
                         'filter': 'G'
                     }
 
-                    rd, created = ReducedDatum.objects.get_or_create(
+                    rd, _ = ReducedDatum.objects.get_or_create(
                         timestamp=jd.to_datetime(timezone=TimezoneInfo()),
-                        value=json.dumps(value),
+                        value=value,
                         source_name=self.name,
                         source_location=alert_url,
                         data_type='photometry',
                         target=target)
-                    rd.save()
 
         return
