@@ -1,9 +1,10 @@
 import requests
-from urllib.parse import urlencode
 from dateutil.parser import parse
+from urllib.parse import urlencode
+
 from astropy import units as u
 from astropy.coordinates import Angle
-
+from crispy_forms.layout import HTML, Layout
 
 from tom_alerts.alerts import GenericAlert, GenericQueryForm, GenericBroker
 from tom_targets.models import Target
@@ -12,7 +13,20 @@ SCOUT_URL = 'https://ssd-api.jpl.nasa.gov/scout.api'
 
 
 class ScoutQueryForm(GenericQueryForm):
-    pass
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.helper.layout = Layout(
+            HTML('''
+                <p>
+                Please see the <a href="https://ssd-api.jpl.nasa.gov/doc/scout.html" target="_blank">Scout API
+                Reference</a> for a detailed description of the service.
+                </br>
+                <b>This form is a placeholder for future broker development. This query will currently retrieve all
+                 available summary data.</b>
+                </p>
+            '''),
+            self.common_layout,
+        )
 
 
 def hours_min_to_decimal(val):
@@ -22,6 +36,11 @@ def hours_min_to_decimal(val):
 
 
 class ScoutBroker(GenericBroker):
+    """
+    The ``ScoutBroker`` is the interface to the Scout alert broker. For information regarding the Scout Broker,
+    please see https://cneos.jpl.nasa.gov/scout/intro.html, as well as https://ssd-api.jpl.nasa.gov/doc/scout.html.
+    """
+
     name = 'Scout'
     form = ScoutQueryForm
 
@@ -32,17 +51,18 @@ class ScoutBroker(GenericBroker):
         return {k.replace('_', '-'): v for k, v in parameters.items() if v}
 
     def fetch_alerts(self, parameters):
+        broker_feedback = ''
         args = urlencode(self.clean_parameters(parameters))
         url = '{0}?{1}'.format(SCOUT_URL, args)
         response = requests.get(url)
         response.raise_for_status()
         parsed = response.json()['data']
         parsed.sort(key=lambda x: parse(x['lastRun']), reverse=True)
-        return iter(parsed)
+        return iter(parsed), broker_feedback
 
-    def fetch_alert(self, id):
-        url = f'{SCOUT_URL}/{id}/?format=json'
-        url = '{0}?tdes={1}'.format(SCOUT_URL, id)
+    def fetch_alert(self, alert_id):
+        url = f'{SCOUT_URL}/{alert_id}/?format=json'
+        url = '{0}?tdes={1}'.format(SCOUT_URL, alert_id)
         response = requests.get(url)
         response.raise_for_status()
         return response.json()
@@ -52,7 +72,6 @@ class ScoutBroker(GenericBroker):
 
     def to_target(self, alert):
         target = Target.objects.create(
-            identifier=alert['objectName'],
             name=alert['objectName'],
             type='NON_SIDEREAL',
             ra=hours_min_to_decimal(alert['ra']),
