@@ -1,4 +1,4 @@
-import json
+from requests import HTTPError
 from unittest.mock import patch
 
 from django import forms
@@ -7,9 +7,10 @@ from django.contrib.messages import get_messages
 from django.core.cache import cache
 from django.test import TestCase, override_settings
 from django.urls import reverse
+from guardian.shortcuts import get_perms
 
 from tom_alerts.alerts import GenericBroker, GenericQueryForm, GenericUpstreamSubmissionForm, GenericAlert
-from tom_alerts.alerts import get_service_class
+from tom_alerts.alerts import get_service_class, get_service_classes
 from tom_alerts.exceptions import AlertSubmissionException
 from tom_alerts.models import BrokerQuery
 from tom_observations.models import ObservationRecord
@@ -50,7 +51,13 @@ class TestBroker(GenericBroker):
     alert_submission_form = TestUpstreamSubmissionForm
 
     def fetch_alerts(self, parameters):
-        """ All brokers must implement this method. It must return a list of alerts.
+        """ All brokers must implement this method. It must return a list of alerts and may include broker feedback.
+        """
+        # Here we simply return a list of `GenericAlert`s that match the name passed in via `parameters`.
+        return iter([alert for alert in test_alerts if alert['name'] == parameters['name']]), "test message"
+
+    def no_message_fetch_alerts(self, parameters):
+        """ Older brokers might implement this version of the fetch_alerts method. It returns a list of alerts.
         """
         # Here we simply return a list of `GenericAlert`s that match the name passed in via `parameters`.
         return iter([alert for alert in test_alerts if alert['name'] == parameters['name']])
@@ -91,7 +98,12 @@ class TestBrokerClass(TestCase):
             get_service_class('MARS')
 
     def test_fetch_alerts(self):
-        alerts = TestBroker().fetch_alerts({'name': 'Hoth'})
+        alerts, _ = TestBroker().fetch_alerts({'name': 'Hoth'})
+        self.assertEqual(test_alerts[1], list(alerts)[0])
+
+    def test_old_fetch_alerts(self):
+        alerts = TestBroker().no_message_fetch_alerts({'name': 'Hoth'})
+        self.assertFalse(isinstance(alerts, tuple))
         self.assertEqual(test_alerts[1], list(alerts)[0])
 
     def test_to_generic_alert(self):
@@ -103,6 +115,25 @@ class TestBrokerClass(TestCase):
         self.assertEqual(target.name, test_alerts[0]['name'])
 
 
+@override_settings(TOM_ALERT_CLASSES=['tom_alerts.fake_broker'])
+class TestAlertModule(TestCase):
+    """Test that attempting to import a nonexistent broker module raises the appropriate errors.
+    """
+
+    def test_get_service_classes_import_error(self):
+        with self.subTest('Invalid import returns an import error.'):
+            with patch('tom_alerts.alerts.import_module') as mock_import_module:
+                mock_import_module.side_effect = ImportError()
+                with self.assertRaisesRegex(ImportError, 'Could not import tom_alerts.fake_broker.'):
+                    get_service_classes()
+
+        with self.subTest('Invalid import returns an attribute error.'):
+            with patch('tom_alerts.alerts.import_module') as mock_import_module:
+                mock_import_module.side_effect = AttributeError()
+                with self.assertRaisesRegex(ImportError, 'Could not import tom_alerts.fake_broker.'):
+                    get_service_classes()
+
+
 @override_settings(TOM_ALERT_CLASSES=['tom_alerts.tests.tests.TestBroker'])
 class TestBrokerViews(TestCase):
     """ Test the views that use the broker classes
@@ -133,12 +164,12 @@ class TestBrokerViews(TestCase):
         broker_query = BrokerQuery.objects.create(
             name='Is it dust?',
             broker='TEST',
-            parameters='{"name": "Alderaan"}',
+            parameters={'name': 'Alderaan'},
         )
         not_found = BrokerQuery.objects.create(
             name='find hoth',
             broker='TEST',
-            parameters='{"name": "Hoth"}',
+            parameters={'name': 'Hoth'},
         )
         response = self.client.get(reverse('tom_alerts:list') + '?name=dust')
         self.assertContains(response, broker_query.name)
@@ -148,7 +179,7 @@ class TestBrokerViews(TestCase):
         broker_query = BrokerQuery.objects.create(
             name='find hoth',
             broker='TEST',
-            parameters='{"name": "Hoth"}',
+            parameters={'name': 'Hoth'},
         )
         self.assertTrue(BrokerQuery.objects.filter(name='find hoth').exists())
         self.client.post(reverse('tom_alerts:delete', kwargs={'pk': broker_query.id}))
@@ -158,16 +189,32 @@ class TestBrokerViews(TestCase):
         broker_query = BrokerQuery.objects.create(
             name='find hoth',
             broker='TEST',
-            parameters='{"name": "Hoth"}',
+            parameters={'name': 'Hoth'},
         )
         response = self.client.get(reverse('tom_alerts:run', kwargs={'pk': broker_query.id}))
         self.assertContains(response,  '66')
 
+    @patch('tom_alerts.tests.tests.TestBroker.fetch_alerts')
+    def test_handle_http_error(self, mock_fetch_alerts):
+        broker_query = BrokerQuery.objects.create(
+            name='find hoth',
+            broker='TEST',
+            parameters={'name': 'Hoth'},
+        )
+        # Set up the mock to raise an HTTPError
+        mock_fetch_alerts.side_effect = HTTPError("Test HTTP Error")
+
+        # Replace 'query_id' with the appropriate identifier for your test query
+        response = self.client.get(reverse('tom_alerts:run', kwargs={'pk': broker_query.id}))
+
+        # Assert that the HTTPError is handled as expected
+        self.assertContains(response, "Issue fetching alerts, please try again.")
+
     def test_update_query(self):
         broker_query = BrokerQuery.objects.create(
             name='find hoth',
             broker='TEST',
-            parameters='{"name": "Hoth"}',
+            parameters={'name': 'Hoth'},
         )
         update_data = {
             'query_name': 'find hoth',
@@ -176,7 +223,7 @@ class TestBrokerViews(TestCase):
         }
         self.client.post(reverse('tom_alerts:update', kwargs={'pk': broker_query.id}), data=update_data)
         broker_query.refresh_from_db()
-        self.assertEqual(broker_query.parameters_as_dict['name'], update_data['name'])
+        self.assertEqual(broker_query.parameters['name'], update_data['name'])
 
     @override_settings(CACHES={
             'default': {
@@ -184,11 +231,11 @@ class TestBrokerViews(TestCase):
             }
         })
     def test_create_target(self):
-        cache.set('alert_2', json.dumps(test_alerts[1]))
+        cache.set('alert_2', test_alerts[1])
         query = BrokerQuery.objects.create(
             name='find hoth',
             broker='TEST',
-            parameters='{"name": "Hoth"}',
+            parameters={'name': 'Hoth'},
         )
         post_data = {
             'broker': 'TEST',
@@ -198,7 +245,9 @@ class TestBrokerViews(TestCase):
         response = self.client.post(reverse('tom_alerts:create-target'), data=post_data)
         self.assertEqual(Target.objects.count(), 1)
         self.assertEqual(Target.objects.first().name, 'Hoth')
-        self.assertRedirects(response, reverse('tom_targets:update', kwargs={'pk': Target.objects.first().id}))
+        self.assertRedirects(response, reverse('tom_targets:list'))
+        # Check Group level permissions
+        self.assertIn("view_target", get_perms(self.user, Target.objects.first()))
 
     @override_settings(CACHES={
             'default': {
@@ -206,12 +255,12 @@ class TestBrokerViews(TestCase):
             }
         })
     def test_create_multiple_targets(self):
-        cache.set('alert_1', json.dumps(test_alerts[0]))
-        cache.set('alert_2', json.dumps(test_alerts[1]))
+        cache.set('alert_1', test_alerts[0])
+        cache.set('alert_2', test_alerts[1])
         query = BrokerQuery.objects.create(
             name='find anything',
             broker='TEST',
-            parameters='{"score__gt": "19"}',
+            parameters={'score__gt': 19},
         )
         post_data = {
             'broker': 'TEST',
@@ -226,7 +275,7 @@ class TestBrokerViews(TestCase):
         query = BrokerQuery.objects.create(
             name='find anything',
             broker='TEST',
-            parameters='{"name": "Alderaan"}',
+            parameters={'name': 'Alderaan'},
         )
         post_data = {
             'broker': 'TEST',
@@ -271,7 +320,6 @@ class TestBrokerViews(TestCase):
         mock_submit_upstream_alert.return_value = False
         response = self.client.post(reverse('tom_alerts:submit-alert', kwargs={'broker': 'TEST'}),
                                     data={'target': target.id})
-
         messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
         self.assertEqual(len(messages), 1)
         self.assertEqual(messages[0][0], 'Unable to submit one or more alerts to TEST. See logs for details.')
