@@ -1,5 +1,7 @@
-import json
+from copy import deepcopy
 import logging
+from requests import HTTPError
+from typing import List
 
 from django.views.generic.edit import DeleteView, FormMixin, FormView, ProcessFormView
 from django.views.generic.base import TemplateView, View
@@ -51,7 +53,7 @@ class BrokerQueryCreateView(LoginRequiredMixin, FormView):
 
         return get_service_class(broker_name).form
 
-    def get_form(self):
+    def get_form(self, form_class=None):
         """
         Returns an instance of the form to be used in this view.
 
@@ -104,7 +106,7 @@ class BrokerQueryUpdateView(LoginRequiredMixin, FormView):
         self.object = self.get_object()
         return get_service_class(self.object.broker).form
 
-    def get_form(self):
+    def get_form(self, form_class=None):
         """
         Returns an instance of the form to be used in this view.
 
@@ -126,7 +128,7 @@ class BrokerQueryUpdateView(LoginRequiredMixin, FormView):
         :rtype: dict
         """
         initial = super().get_initial()
-        initial.update(self.object.parameters_as_dict)
+        initial.update(self.object.parameters)
         initial['broker'] = self.object.broker
         return initial
 
@@ -186,6 +188,30 @@ class RunQueryView(TemplateView):
     """
     template_name = 'tom_alerts/query_result.html'
 
+    def get_template_names(self) -> List[str]:
+        """Override the base class method to ask the broker if it has
+        specified a Broker-specific template to use. If so, put it at the
+        front of the returned list of template_names.
+        """
+        template_names = super().get_template_names()
+
+        # if the broker class has defined a template to use add it to template names (at the front)
+        query = get_object_or_404(BrokerQuery, pk=self.kwargs['pk'])
+        broker_class = get_service_class(query.broker)()
+        logger.debug(f'RunQueryView.get_template_name broker_class: {broker_class}')
+
+        try:
+            if broker_class.template_name:
+                # add to front of list b/c first template will be tried first
+                template_names.insert(0, broker_class.template_name)
+        except AttributeError:
+            # many Brokers won't have a template_name defined and will just
+            # use the one defined above.
+            pass
+
+        logger.debug(f'RunQueryView.get_template_name template_names: {template_names}')
+        return template_names
+
     def get_context_data(self, *args, **kwargs):
         """
         Runs the ``fetch_alerts`` method specific to the given ``BrokerQuery`` and adds the matching alerts to the
@@ -195,21 +221,65 @@ class RunQueryView(TemplateView):
         :rtype: dict
         """
         context = super().get_context_data()
+
+        # get the Broker class
         query = get_object_or_404(BrokerQuery, pk=self.kwargs['pk'])
         broker_class = get_service_class(query.broker)()
-        alerts = broker_class.fetch_alerts(query.parameters_as_dict)
-        context['alerts'] = []
+
+        # Do query and get query results (fetch_alerts)
+        # TODO: Should the deepcopy be in the brokers?
+        try:
+            alert_query_results = broker_class.fetch_alerts(deepcopy(query.parameters))
+
+            # Check if feedback is available for fetch_alerts, and allow for backwards compatibility if not.
+            if isinstance(alert_query_results, tuple):
+                alerts, broker_feedback = alert_query_results
+            else:
+                alerts = alert_query_results
+                broker_feedback = ''
+        except AttributeError:
+            # If the broker isn't configured in settings.py, display error instead of query results
+            alerts = iter(())
+            broker_help = getattr(broker_class, 'help_url',
+                                  'https://tom-toolkit.readthedocs.io/en/latest/api/tom_alerts/brokers.html')
+            broker_feedback = f"""The {broker_class.name} Broker is not properly configured in settings.py.
+                                </br>
+                                Please see the <a href="{broker_help}" target="_blank">documentation</a> for more
+                                information.
+                                """
+        except HTTPError as e:
+            alerts = iter(())
+            broker_feedback = f"Issue fetching alerts, please try again.</br>{e}"
+
+        # Post-query tasks
         query.last_run = timezone.now()
         query.save()
+
+        # create context for template
         context['query'] = query
+        context['score_description'] = broker_class.score_description
+        context['broker_feedback'] = broker_feedback
+        context['too_many_alerts'] = False
+
+        context['alerts'] = []
         try:
-            while len(context['alerts']) < 20:
-                alert = next(alerts)
+            for (i, alert) in enumerate(alerts):
+                if i > 99:
+                    # issue 1172 too many alerts causes the cache to overflow
+                    context['too_many_alerts'] = True
+                    break
                 generic_alert = broker_class.to_generic_alert(alert)
-                cache.set('alert_{}'.format(generic_alert.id), json.dumps(alert), 3600)
+                cache.set(f'alert_{generic_alert.id}', alert, 3600)
                 context['alerts'].append(generic_alert)
         except StopIteration:
             pass
+
+        # allow the Broker to add to the context (besides the query_results)
+        broker_context_additions = broker_class.get_broker_context_data(alerts)
+        context.update(broker_context_additions)
+        # TODO: in python 3.9 we could use the merge operator context |= broker_dict
+        # context |= broker_context_additions
+
         return context
 
 
@@ -237,28 +307,23 @@ class CreateTargetFromAlertView(LoginRequiredMixin, View):
             if not cached_alert:
                 messages.error(request, 'Could not create targets. Try re running the query again.')
                 return redirect(reverse('tom_alerts:run', kwargs={'pk': query_id}))
-            generic_alert = broker_class().to_generic_alert(json.loads(cached_alert))
+            generic_alert = broker_class().to_generic_alert(cached_alert)
             target, extras, aliases = generic_alert.to_target()
             try:
                 target.save(extras=extras, names=aliases)
-                broker_class().process_reduced_data(target, json.loads(cached_alert))
-                for group in request.user.groups.all().exclude(name='Public'):
+                # Give the user access to the target they created
+                target.give_user_access(self.request.user)
+                broker_class().process_reduced_data(target, cached_alert)
+                for group in request.user.groups.all():
                     assign_perm('tom_targets.view_target', group, target)
                     assign_perm('tom_targets.change_target', group, target)
                     assign_perm('tom_targets.delete_target', group, target)
             except IntegrityError:
                 messages.warning(request, f'Unable to save {target.name}, target with that name already exists.')
                 errors.append(target.name)
-        if (len(alerts) == len(errors)):
+        if len(alerts) == len(errors):
             return redirect(reverse('tom_alerts:run', kwargs={'pk': query_id}))
-        elif (len(alerts) == 1):
-            return redirect(reverse(
-                'tom_targets:update', kwargs={'pk': target.id})
-            )
-        else:
-            return redirect(reverse(
-                'tom_targets:list')
-            )
+        return redirect(reverse('tom_targets:list'))
 
 
 class SubmitAlertUpstreamView(LoginRequiredMixin, FormMixin, ProcessFormView, View):
