@@ -24,8 +24,10 @@ def run_hook(name, *args, **kwargs):
 
 
 def target_post_save(target, created):
+    """This hook runs following update of a target."""
     logger.info('Target post save hook: %s created: %s', target, created)
 
 
 def observation_change_state(observation, previous_state):
+    """This hook runs upon the status of an observation changing."""
     logger.info('Observation change state hook: %s from %s to %s', observation, previous_state, observation.status)
