@@ -1,4 +1,4 @@
-from datetime import datetime
+import logging
 import os
 import tempfile
 
@@ -6,13 +6,17 @@ from astropy.io import fits
 from django.conf import settings
 from django.core.files import File
 from django.db import models
+from django.utils import timezone
 from django.core.exceptions import ValidationError
 from fits2image.conversions import fits_to_jpg
 from PIL import Image
+from importlib import import_module
 
-from tom_targets.models import Target
+from tom_targets.base_models import BaseTarget
+from tom_alerts.models import AlertStreamMessage
 from tom_observations.models import ObservationRecord
 
+logger = logging.getLogger(__name__)
 
 try:
     THUMBNAIL_DEFAULT_SIZE = settings.THUMBNAIL_DEFAULT_SIZE
@@ -20,6 +24,15 @@ except AttributeError:
     THUMBNAIL_DEFAULT_SIZE = (200, 200)
 
 
+# Check settings.py for DATA_PRODUCT_TYPES, and provide defaults if not found
+DEFAULT_DATA_TYPE_CHOICES = (('photometry', 'Photometry'), ('spectroscopy', 'Spectroscopy'))
+try:
+    # Pull out tuples from settings.DATA_PRODUCT_TYPES dictionary to build choice fields for DataProduct Types
+    DATA_TYPE_CHOICES = settings.DATA_PRODUCT_TYPES.values()
+except AttributeError:
+    DATA_TYPE_CHOICES = DEFAULT_DATA_TYPE_CHOICES
+
+
 def find_fits_img_size(filename):
     """
     Returns the size of a FITS image, given a valid FITS image file
@@ -70,7 +83,14 @@ def is_fits_image_file(file):
 
 def data_product_path(instance, filename):
     """
-    Returns the TOM-style path for a ``DataProduct`` file. Structure is <target identifier>/<facility>/<filename>.
+    Returns the TOM-style path for a ``DataProduct`` file.
+    Default behavior can be overridden by user in settings.DATA_PRODUCT_PATH
+    DATA_PRODUCT_PATH must be a dot separated method name pointing to a method that takes two arguments:
+    instance: The specific instance of the ``DataProduct`` class.
+    filename: The filename to add to the path.
+    The method must return a string representing the path to the file.
+
+    The default structure is <target identifier>/<facility>/<filename>.
     ``DataProduct`` objects not associated with a facility will save with 'None' as the facility.
 
     :param instance: The specific instance of the ``DataProduct`` class.
@@ -82,11 +102,21 @@ def data_product_path(instance, filename):
     :returns: The TOM-style path of the file
     :rtype: str
     """
-    # Uploads go to MEDIA_ROOT
-    if instance.observation_record is not None:
-        return '{0}/{1}/{2}'.format(instance.target.name, instance.observation_record.facility, filename)
-    else:
-        return '{0}/none/{1}'.format(instance.target.name, filename)
+    try:
+        path_class = settings.DATA_PRODUCT_PATH
+        try:
+            mod_name, class_name = path_class.rsplit('.', 1)
+            mod = import_module(mod_name)
+            clazz = getattr(mod, class_name)
+        except (ImportError, AttributeError):
+            raise ImportError(f'Could not import {path_class}. Did you provide the correct path?')
+        return clazz(instance, filename)
+    except AttributeError:
+        # Uploads go to MEDIA_ROOT
+        if instance.observation_record is not None:
+            return f'{instance.target.name}/{instance.observation_record.facility}/{filename}'
+        else:
+            return f'{instance.target.name}/none/{filename}'
 
 
 class DataProductGroup(models.Model):
@@ -167,7 +197,7 @@ class DataProduct(models.Model):
         null=True,
         help_text='Data product identifier used by the source of the data product.'
     )
-    target = models.ForeignKey(Target, on_delete=models.CASCADE)
+    target = models.ForeignKey(BaseTarget, on_delete=models.CASCADE)
     observation_record = models.ForeignKey(ObservationRecord, null=True, default=None, on_delete=models.CASCADE)
     data = models.FileField(upload_to=data_product_path, null=True, default=None)
     extra_data = models.TextField(blank=True, default='')
@@ -190,8 +220,9 @@ class DataProduct(models.Model):
         Saves the current `DataProduct` instance. Before saving, validates the `data_product_type` against those
         specified in `settings.py`.
         """
-        for dp_type, dp_values in settings.DATA_PRODUCT_TYPES.items():
-            if not self.data_product_type or self.data_product_type == dp_values[0]:
+        # DATA_TYPE_CHOICES from either settings.py or default types: (type, display)
+        for dp_type, _ in DATA_TYPE_CHOICES:
+            if not self.data_product_type or self.data_product_type == dp_type:
                 break
         else:
             raise ValidationError('Not a valid DataProduct type.')
@@ -204,7 +235,8 @@ class DataProduct(models.Model):
         :returns: Display value for a given data_product_type.
         :rtype: str
         """
-        return settings.DATA_PRODUCT_TYPES[self.data_product_type][1]
+        data_product_type_dict = {dp_type: dp_display for dp_type, dp_display in DATA_TYPE_CHOICES}
+        return data_product_type_dict[self.data_product_type]
 
     def get_file_name(self):
         return os.path.basename(self.data.name)
@@ -231,8 +263,9 @@ class DataProduct(models.Model):
         """
         if self.thumbnail:
             im = Image.open(self.thumbnail)
-            if im.size != THUMBNAIL_DEFAULT_SIZE:
+            if im.size != THUMBNAIL_DEFAULT_SIZE and im.size[0] not in THUMBNAIL_DEFAULT_SIZE:
                 redraw = True
+                logger.critical("Redrawing thumbnail for {0} due to size mismatch".format(im.size))
 
         if not self.thumbnail or redraw:
             width, height = THUMBNAIL_DEFAULT_SIZE
@@ -244,7 +277,7 @@ class DataProduct(models.Model):
                     self.thumbnail.save(filename, File(f), save=True)
                     self.save()
         if not self.thumbnail:
-            return ''
+            return None
         return self.thumbnail.url
 
     def create_thumbnail(self, width=None, height=None):
@@ -259,13 +292,19 @@ class DataProduct(models.Model):
         :returns: Thumbnail file if created, None otherwise
         :rtype: file
         """
+        if not self.data:
+            logger.error(f'Unable to create thumbnail for {self}: No data file found.')
+            return
         if is_fits_image_file(self.data.file):
-            tmpfile = tempfile.NamedTemporaryFile()
-            if not width or not height:
-                width, height = find_fits_img_size(self.data.file)
-            resp = fits_to_jpg(self.data.file, tmpfile.name, width=width, height=height)
-            if resp:
-                return tmpfile
+            tmpfile = tempfile.NamedTemporaryFile(suffix='.jpg')
+            try:
+                if not width or not height:
+                    width, height = find_fits_img_size(self.data.file)
+                resp = fits_to_jpg(self.data.file, tmpfile.name, width=width, height=height)
+                if resp:
+                    return tmpfile
+            except Exception as e:
+                logger.warning(f'Unable to create thumbnail for {self}: {e}')
         return
 
 
@@ -296,7 +335,7 @@ class ReducedDatum(models.Model):
     :param timestamp: The timestamp of this datum.
     :type timestamp: datetime
 
-    :param value: The value of the datum. This is generally a JSON string, intended to store data with a variety of
+    :param value: The value of the datum. This is a dict, intended to store data with a variety of
                   scopes. As an example, a photometry value might contain the following:
 
                   ::
@@ -306,41 +345,73 @@ class ReducedDatum(models.Model):
                       'error': .5
                     }
 
-                  but could also contain a filter:
+                  but could also contain a filter, a telescope, an instrument, and/or a unit:
 
                   ::
 
                     {
                       'magnitude': 18.5,
-                      'magnitude_error': .5,
-                      'filter': 'r'
+                      'error': .5,
+                      'filter': 'r',
+                      'telescope': 'ELP.domeA.1m0a',
+                      'instrument': 'fa07',
                     }
+    :type value: dict
 
-                  It should be noted that when storing a dict in a ``ReducedDatum`` value field, it should always be
-                  converted with ``json.dumps`` before saving, as certain functions in the TOM Toolkit call
-                  ``json.loads`` with ``ReducedDatum`` value fields.
-    :type value: str
+    :param message: Set of ``AlertStreamMessage`` objects this object is associated with.
+    :type message: ManyRelatedManager object
 
     """
 
-    target = models.ForeignKey(Target, null=False, on_delete=models.CASCADE)
-    data_product = models.ForeignKey(DataProduct, null=True, on_delete=models.CASCADE)
+    target = models.ForeignKey(BaseTarget, null=False, on_delete=models.CASCADE)
+    data_product = models.ForeignKey(DataProduct, null=True, blank=True, on_delete=models.CASCADE)
     data_type = models.CharField(
         max_length=100,
         default=''
     )
-    source_name = models.CharField(max_length=100, default='')
-    source_location = models.CharField(max_length=200, default='')
-    timestamp = models.DateTimeField(null=False, blank=False, default=datetime.now, db_index=True)
-    value = models.TextField(null=False, blank=False)
+    source_name = models.CharField(max_length=100, default='', blank=True)
+    source_location = models.CharField(max_length=200, default='', blank=True)
+    timestamp = models.DateTimeField(null=False, blank=False, default=timezone.now, db_index=True)
+    value = models.JSONField(null=False, blank=False)
+    message = models.ManyToManyField(AlertStreamMessage, blank=True)
 
     class Meta:
         get_latest_by = ('timestamp',)
 
     def save(self, *args, **kwargs):
-        for dp_type, dp_values in settings.DATA_PRODUCT_TYPES.items():
-            if self.data_type and self.data_type == dp_values[0]:
+        # Validate data_type based on options in settings.py or default types: (type, display)
+        for dp_type, _ in DATA_TYPE_CHOICES:
+            if self.data_type and self.data_type == dp_type:
                 break
         else:
             raise ValidationError('Not a valid DataProduct type.')
+
+        # because we have a custom way of validating the uniqueness of the ReducedDatum,
+        #  we need to call full_clean() here to invoke our validate_unique() method.
+        self.full_clean()
         return super().save()
+
+    def validate_unique(self, *args, **kwargs):
+        """
+        Validates that the ReducedDatum is unique. Because the `value` field is a JSONField, it is not possible to rely
+        on standard validation.
+
+        Do nothing if the uniqueness test passes. Otherwise, raise a ValidationError.
+
+        see https://docs.djangoproject.com/en/5.0/ref/models/instances/#validating-objects
+        """
+        super().validate_unique(*args, **kwargs)
+
+        # Check if the Reduced Datum exists in the database
+        try:
+            existing_reduced_datum = ReducedDatum.objects.get(target=self.target,
+                                                              data_type=self.data_type,
+                                                              timestamp=self.timestamp,
+                                                              value=self.value)
+            if existing_reduced_datum and existing_reduced_datum.id != self.id:  # not the same object
+                # found ReducedDatum with the same values. Don't save this duplicate ReducedDatum.
+                raise ValidationError(f'ReducedDatum already exists: {self.data_type} data with value of {self.value} '
+                                      f'found for {self.target} at {self.timestamp}')
+        except ReducedDatum.DoesNotExist:
+            # this means that our check for uniqueness passed: so do not raise ValidationError
+            pass
