@@ -1,9 +1,9 @@
 import mimetypes
-import json
 
 from astropy import units
-from astropy.io import ascii
+from astropy.io import ascii as astropy_ascii
 from astropy.time import Time, TimezoneInfo
+import numpy as np
 
 from tom_dataproducts.data_processor import DataProcessor
 from tom_dataproducts.exceptions import InvalidFileFormatException
@@ -26,7 +26,7 @@ class PhotometryProcessor(DataProcessor):
         mimetype = mimetypes.guess_type(data_product.data.path)[0]
         if mimetype in self.PLAINTEXT_MIMETYPES:
             photometry = self._process_photometry_from_plaintext(data_product)
-            return [(datum.pop('timestamp'), json.dumps(datum)) for datum in photometry]
+            return [(datum.pop('timestamp'), datum, datum.pop('source', '')) for datum in photometry]
         else:
             raise InvalidFileFormatException('Unsupported file type')
 
@@ -46,7 +46,7 @@ class PhotometryProcessor(DataProcessor):
 
         photometry = []
 
-        data = ascii.read(data_product.data.path)
+        data = astropy_ascii.read(data_product.data.path)
         if len(data) < 1:
             raise InvalidFileFormatException('Empty table or invalid file type')
 
@@ -56,10 +56,10 @@ class PhotometryProcessor(DataProcessor):
             time.format = 'datetime'
             value = {
                 'timestamp': time.to_datetime(timezone=utc),
-                'magnitude': datum['magnitude'],
-                'filter': datum['filter'],
-                'error': datum['error']
             }
+            for column_name in datum.colnames:
+                if not np.ma.is_masked(datum[column_name]):
+                    value[column_name] = datum[column_name]
             photometry.append(value)
 
         return photometry
