@@ -1,7 +1,5 @@
 from abc import ABC, abstractmethod
-from importlib import import_module
 import copy
-import json
 import logging
 import requests
 
@@ -11,16 +9,16 @@ from django import forms
 from django.conf import settings
 from django.contrib.auth.models import Group
 from django.core.files.base import ContentFile
+from django.utils.module_loading import import_string
 
 from tom_targets.models import Target
 
 logger = logging.getLogger(__name__)
 
 DEFAULT_FACILITY_CLASSES = [
-        'tom_observations.facilities.lco.LCOFacility',
-        'tom_observations.facilities.gemini.GEMFacility',
-        'tom_observations.facilities.soar.SOARFacility',
-        'tom_observations.facilities.lt.LTFacility'
+    'tom_observations.facilities.lco.LCOFacility',
+    'tom_observations.facilities.gemini.GEMFacility',
+    'tom_observations.facilities.soar.SOARFacility',
 ]
 
 try:
@@ -37,12 +35,10 @@ def get_service_classes():
 
     service_choices = {}
     for service in TOM_FACILITY_CLASSES:
-        mod_name, class_name = service.rsplit('.', 1)
         try:
-            mod = import_module(mod_name)
-            clazz = getattr(mod, class_name)
+            clazz = import_string(service)
         except (ImportError, AttributeError):
-            raise ImportError('Could not import {}. Did you provide the correct path?'.format(service))
+            raise ImportError(f'Could not import {service}. Did you provide the correct path?')
         service_choices[clazz.name] = clazz
     return service_choices
 
@@ -70,6 +66,7 @@ class BaseObservationForm(forms.Form):
     observation_type = forms.CharField(required=False, max_length=50, widget=forms.HiddenInput())
 
     def __init__(self, *args, **kwargs):
+        self.validation_message = 'This observation is valid.'
         super().__init__(*args, **kwargs)
         self.helper = FormHelper()
         if settings.TARGET_PERMISSIONS_ONLY:
@@ -85,25 +82,38 @@ class BaseObservationForm(forms.Form):
             self.button_layout()
         )
 
-    def layout(self):
+    def layout(self) -> Layout:
+        """Define (and return) a crispy_forms.Layout for the fields of your subclass.
+        It will be inserted after the common_layout and before the button_layout, as
+        defined above in __init__(), where self.helper.layout is assigned.
+
+        This method should be implemented in your subclass to define the layout of your form fields.
+        """
         return
 
     def button_layout(self):
         target_id = self.initial.get('target_id')
         return ButtonHolder(
-                Submit('submit', 'Submit'),
-                HTML(f'''<a class="btn btn-outline-primary" href={{% url 'tom_targets:detail' {target_id} %}}>
-                         Back</a>''')
-            )
+            Submit('submit', 'Submit'),
+            Submit('validate', 'Validate'),
+            HTML(f'''<a class="btn btn-outline-primary"
+             href="{{% url 'tom_targets:detail' {target_id} %}}?tab=observe">Back</a>''')
+        )
+
+    def get_validation_message(self):
+        """ Override this or self.validation_message to return a validation message that is shown when
+            the Validate button is clicked and the form is valid
+        """
+        return self.validation_message
 
     def is_valid(self):
         # TODO: Make this call the validate_observation method in facility
         return super().is_valid()
 
-    def serialize_parameters(self):
+    def serialize_parameters(self) -> dict:
         parameters = copy.deepcopy(self.cleaned_data)
         parameters.pop('groups', None)
-        return json.dumps(parameters)
+        return parameters
 
     def observation_payload(self):
         """
@@ -133,7 +143,6 @@ class BaseRoboticObservationForm(BaseObservationForm):
     For an implementation example please see
     https://github.com/TOMToolkit/tom_base/blob/main/tom_observations/facilities/lco.py#L132
     """
-    pass
 
 
 # This aliasing exists to support backwards compatibility
@@ -181,6 +190,13 @@ class BaseObservationFacility(ABC):
     the other BaseObservationFacilities.
     """
     name = 'BaseObservation'
+    observation_forms = {}
+
+    def __init__(self):
+        self.user = None
+
+    def set_user(self, user):
+        self.user = user
 
     def all_data_products(self, observation_record):
         from tom_dataproducts.models import DataProduct
@@ -211,7 +227,25 @@ class BaseObservationFacility(ABC):
         """
         This method takes in an observation type and returns the form type that matches it.
         """
-        pass
+
+    def get_form_classes_for_display(self, **kwargs):
+        """
+        This method returns a dictionary of the format:
+
+            {'OBSERVATION_TYPE': FacilityFormClass}
+
+        Typically this will be all or a subset of the forms in `self.observation_forms`
+        """
+        return self.observation_forms
+
+    def get_facility_context_data(self, **kwargs):
+        """
+        This method provides an opportunity for the Facility subclass to add additional
+        context data. It will be called by the OberservationCreateView.get_context_data()
+        method and the context dictionary passed to the template will be updated with the
+        returned facility context dictionary.
+        """
+        return kwargs
 
     # TODO: consider making submit_observation create ObservationRecords as well
     @abstractmethod
@@ -220,27 +254,25 @@ class BaseObservationFacility(ABC):
         This method takes in the serialized data from the form and actually
         submits the observation to the remote api
         """
-        pass
 
     @abstractmethod
     def validate_observation(self, observation_payload):
         """
         Same thing as submit_observation, but a dry run. You can
         skip this in different modules by just using "pass"
+
+        Typically called by the ObservationForm.is_valid() method.
         """
-        pass
 
     def get_flux_constant(self):
         """
         Returns the astropy quantity that a facility uses for its spectral flux conversion.
         """
-        pass
 
     def get_wavelength_units(self):
         """
         Returns the astropy units that a facility uses for its spectral wavelengths
         """
-        pass
 
     def is_fits_facility(self, header):
         """
@@ -262,7 +294,6 @@ class BaseObservationFacility(ABC):
         Returns the states for which an observation is not expected
         to change.
         """
-        pass
 
     @abstractmethod
     def get_observing_sites(self):
@@ -273,7 +304,6 @@ class BaseObservationFacility(ABC):
         latitude, longitude and elevation. This is the static information
         about a site.
         """
-        pass
 
     def get_facility_weather_urls(self):
         """
@@ -308,6 +338,14 @@ class BaseObservationFacility(ABC):
         """
         return {}
 
+    def cancel_observation(self, observation_id):
+        """
+        Takes an observation id and submits a request to the observatory that the observation be cancelled.
+
+        If the cancellation was successful, return True. Otherwise, return False.
+        """
+        raise NotImplementedError('This facility has not implemented cancel observation.')
+
     @abstractmethod
     def get_observation_url(self, observation_id):
         """
@@ -316,7 +354,9 @@ class BaseObservationFacility(ABC):
         we return a URL to the LCO observation portal's observation
         record page.
         """
-        pass
+
+    def get_date_obs_from_fits_header(self, header):
+        return None
 
 
 class BaseRoboticObservationFacility(BaseObservationFacility):
@@ -392,7 +432,6 @@ class BaseRoboticObservationFacility(BaseObservationFacility):
         Return the status for a single observation. observation_id should
         be able to be used to retrieve the status from the external service.
         """
-        pass
 
     @abstractmethod
     def data_products(self, observation_id, product_id=None):
@@ -402,7 +441,6 @@ class BaseRoboticObservationFacility(BaseObservationFacility):
         the LCO module retrieves a list of frames from the LCO
         data archive.
         """
-        pass
 
 
 # This aliasing exists to support backwards compatibility
