@@ -1,5 +1,3 @@
-import json
-
 from crispy_forms.helper import FormHelper
 from crispy_forms.layout import Layout, Submit
 from django import forms
@@ -23,9 +21,6 @@ class GenericTemplateForm(forms.Form):
         self.helper.add_input(Submit('submit', 'Submit'))
         self.common_layout = Layout('facility', 'template_name')
 
-    def serialize_parameters(self):
-        return json.dumps(self.cleaned_data)
-
     def save(self, template_id=None):
         if template_id:
             template = ObservationTemplate.objects.get(id=template_id)
@@ -33,7 +28,7 @@ class GenericTemplateForm(forms.Form):
             template = ObservationTemplate()
         template.name = self.cleaned_data['template_name']
         template.facility = self.cleaned_data['facility']
-        template.parameters = self.serialize_parameters()
+        template.parameters = self.cleaned_data
         template.save()
         return template
 
@@ -48,10 +43,6 @@ class ApplyObservationTemplateForm(forms.Form):
         choices=[('', '')] + [(k, k) for k in get_cadence_strategies().keys()],
         required=False
     )
-    cadence_frequency = forms.IntegerField(
-        required=False,
-        help_text='Frequency of observations, in hours'
-    )
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -60,7 +51,6 @@ class ApplyObservationTemplateForm(forms.Form):
             'target',
             'observation_template',
             'cadence_strategy',
-            'cadence_frequency'
         )
         self.helper.form_method = 'GET'
-        self.helper.add_input(Submit('run', 'Run'))
+        self.helper.add_input(Submit('run', 'Apply'))
