@@ -19,6 +19,15 @@ from tom_targets.models import Target
 register = template.Library()
 
 
+@register.inclusion_tag('tom_observations/partials/update_status_button.html', takes_context=True)
+def update_status_button(context):
+    """
+    An inelegant way of passing filters in query parameters while updating observation statuses.
+    Used in ObservationListView to retain filters.
+    """
+    return {'query_params': urlencode(context['request'].GET.dict())}
+
+
 @register.filter
 def display_obs_type(value):
     """
@@ -70,25 +79,26 @@ def observation_type_tabs(context):
     }
 
 
-@register.inclusion_tag('tom_observations/partials/facility_observation_form.html')
-def facility_observation_form(target, facility, observation_type):
+@register.inclusion_tag('tom_observations/partials/facility_observation_form.html', takes_context=True)
+def facility_observation_form(context, target, facility, observation_type):
     """
     Displays a form for submitting an observation for a specific facility and observation type, e.g., imaging.
     """
-    facility_class = get_service_class(facility)()
+    facility_instance = get_service_class(facility)()
+    facility_instance.set_user(context['request'].user)
     initial_fields = {
         'target_id': target.id,
         'facility': facility,
         'observation_type': observation_type
     }
-    obs_form = facility_class.get_form(observation_type)(initial=initial_fields)
+    obs_form = facility_instance.get_form(observation_type)(initial=initial_fields)
     obs_form.helper.form_action = reverse('tom_observations:create', kwargs={'facility': facility})
 
     return {'obs_form': obs_form}
 
 
 @register.inclusion_tag('tom_observations/partials/observation_plan.html')
-def observation_plan(target, facility, length=7, interval=60, airmass_limit=None):
+def observation_plan(target, facility=None, length=2, interval=60, airmass_limit=None):
     """
     Displays form and renders plot for visibility calculation. Using this templatetag to render a plot requires that
     the context of the parent view have values for start_time, end_time, and airmass.
@@ -98,7 +108,7 @@ def observation_plan(target, facility, length=7, interval=60, airmass_limit=None
     start_time = datetime.now()
     end_time = start_time + timedelta(days=length)
 
-    visibility_data = get_sidereal_visibility(target, start_time, end_time, interval, airmass_limit)
+    visibility_data = get_sidereal_visibility(target, start_time, end_time, interval, airmass_limit, facility)
     i = 0
     plot_data = []
     for site, data in visibility_data.items():
@@ -151,7 +161,7 @@ def observationtemplate_from_record(obsr):
     Renders a button that will pre-populate and observation template form with parameters from the specified
     ``ObservationRecord``.
     """
-    obs_params = obsr.parameters_as_dict
+    obs_params = obsr.parameters
     obs_params.pop('target_id', None)
     template_params = urlencode(obs_params)
     return {
@@ -250,8 +260,8 @@ def observation_distribution(observations):
     return {'figure': figure}
 
 
-@register.inclusion_tag('tom_observations/partials/facility_status.html')
-def facility_status():
+@register.inclusion_tag('tom_observations/partials/facility_status.html', takes_context=True)
+def facility_status(context):
     """
     Collect the facility status from the registered facilities and pass them
     to the facility_status.html partial template.
@@ -260,8 +270,9 @@ def facility_status():
     """
 
     facility_statuses = []
-    for _, facility_class in get_service_classes().items():
+    for facility_class in get_service_classes().values():
         facility = facility_class()
+        facility.set_user(context['request'].user)
         weather_urls = facility.get_facility_weather_urls()
         status = facility.get_facility_status()
 
@@ -275,3 +286,52 @@ def facility_status():
         facility_statuses.append(status)
 
     return {'facilities': facility_statuses}
+
+
+@register.inclusion_tag('tom_observations/partials/facility_map.html', takes_context=True)
+def facility_map(context):
+    facility_locations = []
+    for facility_class in get_service_classes().values():
+        facility = facility_class()
+        facility.set_user(context['request'].user)
+        sites = facility.get_observing_sites()
+
+        # Flatten each facility site dictionary and add text label for use in facility map
+        # Resulting list is of the format [['LCO', 'Siding Spring', 'coj', -31.272, 149.07, 1116], ...]
+        facility_locations.extend([
+            [facility.name, site_name] + [value for value in site_data.values()]
+            for site_name, site_data in sites.items()
+        ])
+
+    data = [
+        dict(
+            lat=[site[3] for site in facility_locations],
+            lon=[site[4] for site in facility_locations],
+            text=[f'{site[0]}: {site[1]}' for site in facility_locations],
+            hoverinfo='text',
+            mode='markers',
+            type='scattergeo'
+        )
+    ]
+    layout = {
+        'title': 'Facility Sites',
+        'hovermode': 'closest',
+        'showlegend': False,
+        'geo': {
+            'projection': {
+                'type': 'mollweide',
+            },
+            'showcoastlines': True,
+            'showland': True,
+            'lonaxis': {
+                'showgrid': True,
+                'range': [0, 360],
+            },
+            'lataxis': {
+                'showgrid': True,
+                'range': [-90, 90],
+            },
+        }
+    }
+    figure = offline.plot(go.Figure(data=data, layout=layout), output_type='div', show_link=False)
+    return {'figure': figure}
