@@ -1,5 +1,4 @@
 import factory
-import json
 
 from tom_targets.models import Target, TargetName
 from tom_observations.models import ObservationRecord, ObservationTemplate
@@ -52,7 +51,7 @@ class ObservingRecordFactory(factory.django.DjangoModelFactory):
     facility = 'LCO'
     observation_id = factory.Faker('pydecimal', right_digits=0, left_digits=7)
     status = 'PENDING'
-    parameters = json.dumps({
+    parameters = {
         'facility': 'LCO',
         'target_id': 1,
         'observation_type': 'IMAGING',
@@ -67,7 +66,7 @@ class ObservingRecordFactory(factory.django.DjangoModelFactory):
         'proposal': 'LCOSchedulerTest',
         'filter': 'I',
         'instrument_type': '1M0-SCICAM-SINISTRO'
-    })
+    }
 
 
 class ObservationTemplateFactory(factory.django.DjangoModelFactory):
@@ -75,7 +74,7 @@ class ObservationTemplateFactory(factory.django.DjangoModelFactory):
         model = ObservationTemplate
 
     facility = 'LCO'
-    parameters = json.dumps({
+    parameters = {
         'facility': 'LCO',
         'target_id': 1,
         'observation_type': 'IMAGING',
@@ -90,4 +89,4 @@ class ObservationTemplateFactory(factory.django.DjangoModelFactory):
         'proposal': 'LCOSchedulerTest',
         'filter': 'I',
         'instrument_type': '1M0-SCICAM-SINISTRO'
-    })
+    }
