@@ -1,9 +1,13 @@
 from datetime import datetime, timedelta
+from http import HTTPStatus
 from unittest import mock
 
+from django.contrib.auth.models import User
+from django.contrib.messages import get_messages
+from django.forms import ValidationError
 from django.test import TestCase, override_settings
 from django.urls import reverse
-from django.contrib.auth.models import User
+from django.utils import timezone
 
 from astroplan import FixedTarget
 from astropy.coordinates import get_sun, SkyCoord
@@ -17,8 +21,7 @@ from tom_targets.models import Target
 from guardian.shortcuts import assign_perm
 
 
-@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility',
-                                         'tom_observations.tests.utils.FakeManualFacility'],
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
                    TARGET_PERMISSIONS_ONLY=True)
 class TestObservationViews(TestCase):
     def setUp(self):
@@ -27,7 +30,7 @@ class TestObservationViews(TestCase):
         self.observation_record = ObservingRecordFactory.create(
             target_id=self.target.id,
             facility=FakeRoboticFacility.name,
-            parameters='{}'
+            parameters={}
         )
         self.user = User.objects.create_user(username='vincent_adultman', password='important')
         self.user2 = User.objects.create_user(username='peon', password='plebian')
@@ -67,8 +70,12 @@ class TestObservationViews(TestCase):
 
     def test_update_observations(self):
         response = self.client.get(reverse('tom_observations:list') + '?update_status=True', follow=True)
-        self.assertEqual(response.status_code, 200)
-        self.assertContains(response, 'COMPLETED')
+        self.assertContains(response, 'COMPLETED', status_code=200)
+
+    def test_update_observations_not_authenticated(self):
+        """Test that an unauthenticated user is redirected to login screen if they attempt to update observations."""
+        response = self.client.get(reverse('tom_observations:list') + '?update_status=True')
+        self.assertEqual(response.status_code, 302)
 
     def test_get_observation_form(self):
         url = f"{reverse('tom_observations:create', kwargs={'facility': 'FakeRoboticFacility'})}" \
@@ -102,6 +109,24 @@ class TestObservationViews(TestCase):
         obs_group.refresh_from_db()
         self.assertNotIn(self.observation_record, obs_group.observation_records.all())
 
+
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility',
+                                         'tom_observations.tests.utils.FakeManualFacility'],
+                   TARGET_PERMISSIONS_ONLY=True)
+class TestObservationCreateView(TestCase):
+    def setUp(self):
+        self.target = SiderealTargetFactory.create()
+        self.target_name = TargetNameFactory.create(target=self.target)
+        self.observation_record = ObservingRecordFactory.create(
+            target_id=self.target.id,
+            facility=FakeRoboticFacility.name,
+            parameters={}
+        )
+        self.user = User.objects.create_user(username='vincent_adultman', password='important')
+        self.user2 = User.objects.create_user(username='peon', password='plebian')
+        assign_perm('tom_targets.view_target', self.user, self.target)
+        self.client.force_login(self.user)
+
     def test_submit_observation_robotic(self):
         form_data = {
             'target_id': self.target.id,
@@ -120,6 +145,36 @@ class TestObservationViews(TestCase):
         self.assertTrue(ObservationRecord.objects.filter(observation_id='fakeid').exists())
         self.assertEqual(ObservationRecord.objects.filter(observation_id='fakeid').first().user, self.user)
 
+    @mock.patch('tom_observations.tests.utils.FakeRoboticFacility.set_user')
+    def test_submit_observation_robotic_gets_user(self, mock_method):
+        form_data = {
+            'target_id': self.target.id,
+            'test_input': 'gnomes',
+            'facility': 'FakeRoboticFacility',
+            'observation_type': 'OBSERVATION'
+        }
+        self.client.post(
+            '{}?target_id={}'.format(
+                reverse('tom_observations:create', kwargs={'facility': 'FakeRoboticFacility'}),
+                self.target.id
+            ),
+            data=form_data,
+            follow=True
+        )
+        calls = [mock.call(self.user)]
+        mock_method.assert_has_calls(calls)
+
+    # TODO: this test
+    # def test_submit_observation_cadence(self):
+    #     form_data = {
+    #         'target_id': self.target.id,
+    #         'test_input': 'gnomes',
+    #         'facility': 'FakeRoboticFacility',
+    #         'observation_type': 'OBSERVATION',
+    #         'cadence_strategy': 'RetryFailedObservationsStrategy',
+    #         'cadence_frequency': 24,
+    #     }
+
     def test_submit_observation_manual(self):
         form_data = {
             'target_id': self.target.id,
@@ -133,6 +188,107 @@ class TestObservationViews(TestCase):
         self.assertEqual(ObservationRecord.objects.filter(observation_id='fakeid').first().user, self.user)
 
 
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                   TARGET_PERMISSIONS_ONLY=True)
+class TestObservationCancelView(TestCase):
+    def setUp(self):
+        self.target = SiderealTargetFactory.create()
+        self.observation_record = ObservingRecordFactory.create(
+            target_id=self.target.id,
+            facility=FakeRoboticFacility.name,
+            parameters={},
+            status='PENDING'
+        )
+        self.user = User.objects.create_user(username='vincent_adultman', password='important')
+        self.client.force_login(self.user)
+
+    @mock.patch('tom_observations.tests.utils.FakeRoboticFacility.get_observation_status')
+    def test_cancel_observation(self, mock_get_status):
+        mock_get_status.return_value = {'state': 'CANCELED',
+                                        'scheduled_start': timezone.now(),
+                                        'scheduled_end': timezone.now()}
+        self.observation_record.status = 'PENDING'
+        self.observation_record.save()
+
+        self.client.get(reverse('tom_observations:cancel', kwargs={'pk': self.observation_record.id}))
+        self.observation_record.refresh_from_db()
+        self.assertEqual(self.observation_record.status, 'CANCELED')
+
+    @mock.patch('tom_observations.tests.utils.FakeRoboticFacility.cancel_observation')
+    def test_cancel_observation_failure(self, mock_cancel_observation):
+        mock_cancel_observation.return_value = False
+        response = self.client.get(reverse('tom_observations:cancel', kwargs={'pk': self.observation_record.id}))
+
+        messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
+        self.assertEqual(messages[0][0], 'Unable to cancel observation.')
+
+    @mock.patch('tom_observations.tests.utils.FakeRoboticFacility.cancel_observation')
+    def test_cancel_observation_exception(self, mock_cancel_observation):
+        mock_cancel_observation.side_effect = ValidationError('mock error')
+        response = self.client.get(reverse('tom_observations:cancel', kwargs={'pk': self.observation_record.id}))
+
+        messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
+        self.assertEqual(messages[0][0], 'Unable to cancel observation: [\'mock error\']')
+
+
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
+                   TARGET_PERMISSIONS_ONLY=True)
+class TestAddExistingObservationView(TestCase):
+    def setUp(self):
+        self.target = SiderealTargetFactory.create(permissions='PUBLIC')
+        self.user = User.objects.create_user(username='vincent_adultman', password='important')
+        self.client.force_login(self.user)
+
+    def test_add_existing_observation(self):
+        form_data = {
+            'facility': 'FakeRoboticFacility',
+            'target_id': self.target.id,
+            'observation_id': '1234567890'
+        }
+        response = self.client.post(reverse('tom_observations:add-existing'), data=form_data, follow=True)
+
+        messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
+        self.assertEqual(messages[0][0], 'Successfully associated observation record 1234567890')
+
+        self.assertTrue(ObservationRecord.objects.filter(observation_id=form_data['observation_id']).exists())
+
+    def test_add_existing_observation_duplicate(self):
+        obsr = ObservingRecordFactory.create(
+            target_id=self.target.id,
+            facility=FakeRoboticFacility.name,
+            parameters={},
+            observation_id='1234567890'
+        )
+
+        form_data = {
+            'facility': 'FakeRoboticFacility',
+            'target_id': self.target.id,
+            'observation_id': obsr.observation_id
+        }
+        response = self.client.post(reverse('tom_observations:add-existing'), data=form_data, follow=True)
+        self.assertContains(response,
+                            'An observation record already exists in your TOM for this combination')
+        self.assertEqual(ObservationRecord.objects.filter(observation_id=obsr.observation_id).count(), 1)
+
+        form_data['confirm'] = True
+        response = self.client.post(reverse('tom_observations:add-existing'), data=form_data, follow=True)
+        messages = [(m.message, m.level) for m in get_messages(response.wsgi_request)]
+        self.assertEqual(messages[0][0], f'Successfully associated observation record {obsr.observation_id}')
+
+        self.assertEqual(ObservationRecord.objects.filter(observation_id=obsr.observation_id).count(), 2)
+
+
+@override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'])
+class TestFacilityStatusView(TestCase):
+    def setUp(self):
+        pass
+
+    def test_facility_status(self):
+        response = self.client.get(reverse('tom_observations:facility-status'))
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'coj.domb.1m0a', status_code=HTTPStatus.OK)
+
+
 @override_settings(TOM_FACILITY_CLASSES=['tom_observations.tests.utils.FakeRoboticFacility'],
                    TARGET_PERMISSIONS_ONLY=False)
 class TestObservationViewsRowLevelPermissions(TestCase):
@@ -142,7 +298,7 @@ class TestObservationViewsRowLevelPermissions(TestCase):
         self.observation_record = ObservingRecordFactory.create(
             target_id=self.target.id,
             facility=FakeRoboticFacility.name,
-            parameters='{}'
+            parameters={}
         )
         user = User.objects.create_user(username='vincent_adultman', password='important')
         self.user2 = User.objects.create_user(username='peon', password='plebian')
@@ -229,13 +385,13 @@ class TestUpdatingObservations(TestCase):
     def test_update_all_observations_for_facility(self):
         with mock.patch.object(FakeRoboticFacility, 'update_observation_status') as uos_mock:
             FakeRoboticFacility().update_all_observation_statuses()
-            self.assertEquals(uos_mock.call_count, 2)
+            self.assertEqual(uos_mock.call_count, 2)
 
     # Tests that only the observing records associated with the given target are updated
     def test_update_individual_target_observations_for_facility(self):
         with mock.patch.object(FakeRoboticFacility, 'update_observation_status', return_value='COMPLETED') as uos_mock:
             FakeRoboticFacility().update_all_observation_statuses(target=self.t1)
-            self.assertEquals(uos_mock.call_count, 2)
+            self.assertEqual(uos_mock.call_count, 2)
 
 
 class TestGetVisibility(TestCase):
@@ -259,8 +415,8 @@ class TestGetVisibility(TestCase):
         expected_time_range = [58400.58074074052, 58400.92796296533,
                                58401.27518519014, 58401.62240741495,
                                58401.96962963976, 58402.31685186457]
-        for i in range(0, len(expected_time_range)):
-            self.assertEqual(check_time_range[i], expected_time_range[i])
+        for i, value in enumerate(expected_time_range):
+            self.assertEqual(check_time_range[i], value)
 
     def test_get_astroplan_sun_and_time_small_range(self):
         end = self.start + timedelta(hours=10)
@@ -270,8 +426,8 @@ class TestGetVisibility(TestCase):
         check_time_range = [time.mjd for time in time_range[::20]]
         expected_time_range = [58400.58074074052, 58400.71962963045,
                                58400.85851852037, 58400.997407410294]
-        for i in range(0, len(expected_time_range)):
-            self.assertEqual(check_time_range[i], expected_time_range[i])
+        for i, value in enumerate(expected_time_range):
+            self.assertEqual(check_time_range[i], value)
 
     def test_get_visibility_invalid_target_type(self):
         invalid_target = self.target
@@ -298,5 +454,5 @@ class TestGetVisibility(TestCase):
             1.2895601364316183, 1.3034413026227516, 1.3203684217446099
         ]
         self.assertEqual(len(airmass_data), len(expected_airmass))
-        for i in range(0, len(expected_airmass)):
-            self.assertAlmostEqual(airmass_data[i], expected_airmass[i], places=3)
+        for i, expected_airmass_value in enumerate(expected_airmass):
+            self.assertAlmostEqual(airmass_data[i], expected_airmass_value, places=3)
