@@ -1,15 +1,18 @@
-from django import forms
 from datetime import timedelta
+
+from django import forms
 from django.utils import timezone
 from astropy import units
 
 from tom_observations.facility import BaseRoboticObservationFacility, GenericObservationForm
 from tom_observations.facility import BaseManualObservationFacility
+from tom_observations.models import ObservationRecord
 from tom_observations.observation_template import GenericTemplateForm
 
 # Site data matches built-in pyephem observer data for Los Angeles
 SITES = {
     'Los Angeles': {
+        'sitecode': 'lax',
         'latitude': 34.052222,
         'longitude': -117.756306,
         'elevation': 86.847092
@@ -24,7 +27,7 @@ SITES = {
 
 
 class FakeFacilityForm(GenericObservationForm):
-    test_input = forms.CharField(help_text='fake form input')
+    test_input = forms.CharField(help_text='fake form input', required=True)
 
 
 class FakeFacilityTemplateForm(GenericTemplateForm):
@@ -65,6 +68,12 @@ class FakeRoboticFacility(BaseRoboticObservationFacility):
     def submit_observation(self, payload):
         return ['fakeid']
 
+    def cancel_observation(self, observation_id):
+        obsr = ObservationRecord.objects.get(observation_id=observation_id)
+        obsr.status = 'CANCELED'
+        obsr.save()
+        return True
+
     def get_flux_constant(self):
         return units.erg / units.angstrom
 
@@ -74,6 +83,31 @@ class FakeRoboticFacility(BaseRoboticObservationFacility):
     def validate_observation(self, observation_payload):
         return True
 
+    def get_facility_weather_urls(self):
+        """
+        `facility_weather_urls = {'code': 'XYZ', 'sites': [ site_dict, ... ]}`
+        where
+        `site_dict = {'code': 'XYZ', 'weather_url': 'http://path/to/weather'}`
+        """
+        # TODO: manually add a weather url for tlv
+        facility_weather_urls = {
+            'code': 'FakeRoboticFacility',
+            'sites': [
+                {
+                    'code': site['sitecode'],
+                    'weather_url': f'https://example.com/#/{site["sitecode"]}'
+                }
+                for site in SITES.values()]
+            }
+
+        return facility_weather_urls
+
+    def get_facility_status(self):
+        return {
+            'code': 'LCO',
+            'sites': [{'code': 'coj', 'telescopes': [{'code': 'coj.domb.1m0a', 'status': 'NOT_OK_TO_OPEN'}]}]
+        }
+
 
 class FakeManualFacility(BaseManualObservationFacility):
     name = 'FakeManualFacility'
