@@ -1,20 +1,23 @@
 from io import StringIO
-from urllib.parse import urlparse
+from urllib.parse import urlencode
+import logging
+from typing import List
 
 from crispy_forms.bootstrap import FormActions
 from crispy_forms.layout import HTML, Layout, Submit
 from django import forms
+from django.core.exceptions import BadRequest
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin
 from django.core.management import call_command
-from django_filters import (CharFilter, ChoiceFilter, DateTimeFromToRangeFilter, FilterSet, ModelMultipleChoiceFilter,
-                            OrderingFilter)
+from django_filters import CharFilter, ChoiceFilter, DateTimeFromToRangeFilter, ModelMultipleChoiceFilter
+from django_filters import OrderingFilter, MultipleChoiceFilter, rest_framework
 from django_filters.views import FilterView
 from django.shortcuts import redirect
 from django.urls import reverse, reverse_lazy
 from django.utils.safestring import mark_safe
-from django.views.generic import View
+from django.views.generic import View, TemplateView
 from django.views.generic.detail import DetailView
 from django.views.generic.edit import CreateView, DeleteView, FormView, UpdateView
 from django.views.generic.list import ListView
@@ -24,15 +27,18 @@ from guardian.mixins import PermissionListMixin
 from tom_common.hints import add_hint
 from tom_common.mixins import Raise403PermissionRequiredMixin
 from tom_dataproducts.forms import AddProductToGroupForm, DataProductUploadForm
+from tom_dataproducts.models import is_fits_image_file
 from tom_observations.cadence import CadenceForm, get_cadence_strategy
 from tom_observations.facility import get_service_class, get_service_classes
 from tom_observations.facility import BaseManualObservationFacility
-from tom_observations.forms import AddExistingObservationForm
+from tom_observations.forms import AddExistingObservationForm, facility_choices
 from tom_observations.models import ObservationRecord, ObservationGroup, ObservationTemplate, DynamicCadence
 from tom_targets.models import Target
 
+logger = logging.getLogger(__name__)
 
-class ObservationFilter(FilterSet):
+
+class ObservationFilter(rest_framework.FilterSet):
     """
     Defines the available fields for filtering the list of ``ObservationRecord`` objects.
     """
@@ -40,10 +46,28 @@ class ObservationFilter(FilterSet):
         fields=['scheduled_start', 'scheduled_end', 'status', 'created', 'modified']
     )
     scheduled_start = DateTimeFromToRangeFilter()
-    scheduled_end = DateTimeFromToRangeFilter
+    scheduled_end = DateTimeFromToRangeFilter()
+    target_id = ModelMultipleChoiceFilter(
+        queryset=Target.objects.filter(observationrecord__isnull=False).distinct().order_by('name')
+    )
     observationgroup = ModelMultipleChoiceFilter(
         label='Observation Groups', queryset=ObservationGroup.objects.all()
     )
+    facility = MultipleChoiceFilter(choices=facility_choices())
+
+    def __init__(self, *args, **kwargs):
+        """
+        The "status" filter is populated dynamically via list comprehension here in the __init__ (at runtime).
+        This is important because the `ObservationRecord` db table doesn't necessarily exist at
+        Class-interpretation-time
+        """
+        super().__init__(*args, **kwargs)
+        self.status = MultipleChoiceFilter(
+            choices=[
+                (s, s) for s in
+                ObservationRecord.objects.values_list('status', flat=True).order_by('status').distinct()
+            ]
+        )
 
     class Meta:
         model = ObservationRecord
@@ -69,7 +93,7 @@ class ObservationListView(FilterView):
         """
         if settings.TARGET_PERMISSIONS_ONLY:
             return ObservationRecord.objects.filter(
-                target__in=get_objects_for_user(self.request.user, 'tom_targets.view_target')
+                target__in=get_objects_for_user(self.request.user, f'{Target._meta.app_label}.view_target')
             )
         else:
             return get_objects_for_user(self.request.user, 'tom_observations.view_observationrecord')
@@ -82,7 +106,9 @@ class ObservationListView(FilterView):
         :param request: request object for this GET request
         :type request: HTTPRequest
         """
-        update_status = request.GET.get('update_status', False)
+        # QueryDict is immutable, and we want to append the remaining parameters to the redirect URL
+        query_params = request.GET.copy()
+        update_status = query_params.pop('update_status', False)
         if update_status:
             if not request.user.is_authenticated:
                 return redirect(reverse('login'))
@@ -93,7 +119,7 @@ class ObservationListView(FilterView):
                               'Did you know updating observation statuses can be automated? Learn how in '
                               '<a href=https://tom-toolkit.readthedocs.io/en/stable/customization/automation.html>'
                               'the docs.</a>'))
-            return redirect(reverse('tom_observations:list'))
+            return redirect(f'{reverse("tom_observations:list")}?{urlencode(query_params)}')
 
         selected = request.GET.getlist('selected')
         observationgroups = request.GET.getlist('observationgroup')
@@ -118,6 +144,27 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
     """
     template_name = 'tom_observations/observation_form.html'
 
+    def get_template_names(self) -> List[str]:
+        """Override the base class method to ask the Facility if it has
+        specified a Facility-specific template to use. If so, put it at the
+        front of the returned list of template_names.
+        """
+        template_names = super().get_template_names()
+
+        # get the facility_class and its template_name, if defined
+        facility_class = self.get_facility_class()
+        try:
+            if facility_class.template_name:
+                # add to front of list b/c first template will be tried first
+                template_names.insert(0, facility_class.template_name)
+        except AttributeError:
+            # some Facilities won't have a custom template_name defined and so
+            # we will just use the one defined above.
+            pass
+
+        logger.debug(f'ObservationCreateView.get_template_name template_names: {template_names}')
+        return template_names
+
     def get_target_id(self):
         """
         Parses the target id for the given observation from the query parameters.
@@ -176,7 +223,10 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
         # reloaded due to form errors, only repopulate the form that was submitted.
         observation_type_choices = []
         initial = self.get_initial()
-        for observation_type, observation_form_class in self.get_facility_class().observation_forms.items():
+        facility = self.get_facility_class()()
+        facility.set_user(self.request.user)
+        observation_form_classes = facility.get_form_classes_for_display(**kwargs)
+        for observation_type, observation_form_class in observation_form_classes.items():
             form_data = {**initial, **{'observation_type': observation_type}}
             # Repopulate the appropriate form with form data if the original submission was invalid
             if observation_type == self.request.POST.get('observation_type'):
@@ -191,6 +241,18 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
 
         target = Target.objects.get(pk=self.get_target_id())
         context['target'] = target
+
+        # allow the Facility class to add data to the context
+        facility_context = facility.get_facility_context_data(target=target)
+        context.update(facility_context)
+
+        context['facility_link'] = getattr(facility, 'link', '')
+
+        try:
+            context['missing_configurations'] = ", ".join(facility.facility_settings.get_unconfigured_settings())
+        except AttributeError:
+            context['missing_configurations'] = ''
+
         return context
 
     def get_form_class(self):
@@ -205,19 +267,28 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
             observation_type = self.request.GET.get('observation_type')
         elif self.request.method == 'POST':
             observation_type = self.request.POST.get('observation_type')
+        facility = self.get_facility_class()()
+        facility.set_user(self.request.user)
         form_class = type(f'Composite{observation_type}Form',
-                          (self.get_facility_class()().get_form(observation_type), self.get_cadence_strategy_form()),
+                          (facility.get_form(observation_type), self.get_cadence_strategy_form()),
                           {})
         return form_class
 
-    def get_form(self):
+    def get_form(self, form_class=None):
         """
         Gets an instance of the form appropriate for the request.
 
         :returns: observation form
         :rtype: subclass of GenericObservationForm
         """
-        form = super().get_form()
+        try:
+            form = super().get_form()
+        except Exception as ex:
+            logger.error(f"Error loading {self.get_facility()} form: {repr(ex)}")
+            raise BadRequest(f"Error loading {self.get_facility()} form: {repr(ex)}")
+
+        # tom_observations/facility.BaseObservationForm.__init__ to see how
+        # groups is added to common_layout
         if not settings.TARGET_PERMISSIONS_ONLY:
             form.fields['groups'].queryset = self.request.user.groups.all()
         form.helper.form_action = reverse(
@@ -244,10 +315,17 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
     def post(self, request, *args, **kwargs):
         form = self.get_form()
         if form.is_valid():
-            return self.form_valid(form)
+            if 'validate' in request.POST:
+                return self.form_validation_valid(form)
+            else:
+                return self.form_valid(form)
         else:
             return self.form_invalid(form)
 
+    def form_validation_valid(self, form):
+        messages.info(self.request, form.get_validation_message())
+        return self.render_to_response(self.get_context_data(request=self.request, form=form))
+
     def form_valid(self, form):
         """
         Runs after form validation. Submits the observation to the desired facility and creates an associated
@@ -260,9 +338,10 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
         :type form: subclass of GenericObservationForm
         """
         # Submit the observation
-        facility = self.get_facility_class()
+        facility = self.get_facility_class()()
+        facility.set_user(self.request.user)
         target = self.get_target()
-        observation_ids = facility().submit_observation(form.observation_payload())
+        observation_ids = facility.submit_observation(form.observation_payload())
         records = []
 
         for observation_id in observation_ids:
@@ -306,7 +385,7 @@ class ObservationCreateView(LoginRequiredMixin, FormView):
                 assign_perm('tom_observations.delete_observationrecord', groups, record)
 
         return redirect(
-            reverse('tom_targets:detail', kwargs={'pk': target.id})
+            reverse('tom_targets:detail', kwargs={'pk': target.id}) + '?tab=observations'
         )
 
 
@@ -322,27 +401,24 @@ class ObservationRecordUpdateView(LoginRequiredMixin, UpdateView):
         return reverse('tom_observations:detail', kwargs={'pk': self.get_object().id})
 
 
-class ObservationGroupCancelView(LoginRequiredMixin, View):
-
-    def get_context_data(self, *args, **kwargs):
-        context = super().get_context_data(*args, **kwargs)
-        context['next'] = self.request.META.get('HTTP_REFERER', '/')
-        return context
+class ObservationRecordCancelView(LoginRequiredMixin, View):
 
     def get(self, request, *args, **kwargs):
         obsr_id = self.kwargs.get('pk')
         obsr = ObservationRecord.objects.get(id=obsr_id)
         facility = get_service_class(obsr.facility)()
-        errors = facility.cancel_observation(obsr.observation_id)
-        if errors:
-            messages.error(
-                self.request,
-                f'Unable to cancel observation: {errors}'
-            )
+        facility.set_user(request.user)
+        try:
+            success = facility.cancel_observation(obsr.observation_id)
+            if success:
+                messages.success(self.request, f'Successfully cancelled observation {obsr}')
+                facility.update_observation_status(obsr.observation_id)
+            else:
+                messages.error(self.request, 'Unable to cancel observation.')
+        except forms.ValidationError as ve:
+            messages.error(self.request, f'Unable to cancel observation: {ve}')
 
-        referer = self.request.GET('next', None)
-        referer = urlparse(referer).path if referer else '/'
-        return redirect(referer)
+        return redirect(reverse('tom_observations:detail', kwargs={'pk': obsr.id}))
 
 
 class AddExistingObservationView(LoginRequiredMixin, FormView):
@@ -360,7 +436,7 @@ class AddExistingObservationView(LoginRequiredMixin, FormView):
     template_name = 'tom_observations/existing_observation_confirm.html'
     form_class = AddExistingObservationForm
 
-    def get_form(self):
+    def get_form(self, form_class=None):
         form = super().get_form()
         form.fields['facility'].widget = forms.HiddenInput()
         form.fields['observation_id'].widget = forms.HiddenInput()
@@ -370,7 +446,7 @@ class AddExistingObservationView(LoginRequiredMixin, FormView):
             target_id = self.request.POST.get('target_id')
         cancel_url = reverse('home')
         if target_id:
-            cancel_url = reverse('tom_targets:detail', kwargs={'pk': target_id})
+            cancel_url = reverse('tom_targets:detail', kwargs={'pk': target_id}) + '?tab=observations'
         form.helper.layout = Layout(
             HTML('''<p>An observation record already exists in your TOM for this combination of observation ID,
                  facility, and target. Are you sure you want to create this record?</p>'''),
@@ -417,9 +493,9 @@ class AddExistingObservationView(LoginRequiredMixin, FormView):
             )
             observation_id = form.cleaned_data['observation_id']
             messages.success(self.request, f'Successfully associated observation record {observation_id}')
-        return redirect(reverse(
-            'tom_targets:detail', kwargs={'pk': form.cleaned_data['target_id']})
-        )
+        base_url = reverse('tom_targets:detail', kwargs={'pk': form.cleaned_data['target_id']})
+        query_params = urlencode({'tab': 'observations'})
+        return redirect(f'{base_url}?{query_params}')
 
 
 class ObservationRecordDetailView(DetailView):
@@ -438,7 +514,7 @@ class ObservationRecordDetailView(DetailView):
         """
         if settings.TARGET_PERMISSIONS_ONLY:
             return ObservationRecord.objects.filter(
-                target__in=get_objects_for_user(self.request.user, 'tom_targets.view_target')
+                target__in=get_objects_for_user(self.request.user, f'{Target._meta.app_label}.view_target')
             )
         else:
             return get_objects_for_user(self.request.user, 'tom_observations.view_observationrecord')
@@ -455,16 +531,17 @@ class ObservationRecordDetailView(DetailView):
         """
         context = super().get_context_data(*args, **kwargs)
         context['form'] = AddProductToGroupForm()
-        service_class = get_service_class(self.object.facility)
-        context['editable'] = isinstance(service_class(), BaseManualObservationFacility)
-        context['data_products'] = service_class().all_data_products(self.object)
-        context['can_be_cancelled'] = self.object.status not in service_class().get_terminal_observing_states()
+        facility = get_service_class(self.object.facility)()
+        facility.set_user(self.request.user)
+        context['editable'] = isinstance(facility, BaseManualObservationFacility)
+        context['data_products'] = facility.all_data_products(self.object)
+        context['can_be_cancelled'] = self.object.status not in facility.get_terminal_observing_states()
         newest_image = None
         for data_product in context['data_products']['saved']:
             newest_image = data_product if (not newest_image or data_product.modified > newest_image.modified) and \
-                data_product.get_file_extension() == '.fits' else newest_image
+                is_fits_image_file(data_product.data.file) else newest_image
         if newest_image:
-            context['image'] = newest_image.get_image_data()
+            context['image'] = newest_image.get_preview()
         data_product_upload_form = DataProductUploadForm(
             initial={
                 'observation_record': self.get_object(),
@@ -517,7 +594,7 @@ class ObservationGroupDeleteView(Raise403PermissionRequiredMixin, DeleteView):
     success_url = reverse_lazy('tom_observations:group-list')
 
 
-class ObservationTemplateFilter(FilterSet):
+class ObservationTemplateFilter(rest_framework.FilterSet):
     """
     Defines the available fields for filtering the list of ``ObservationTemplate`` objects.
     """
@@ -555,6 +632,17 @@ class ObservationTemplateCreateView(FormView):
     def get_facility_name(self):
         return self.kwargs['facility']
 
+    def get_context_data(self, *args, **kwargs):
+        context = super().get_context_data(*args, **kwargs)
+
+        facility = get_service_class(self.get_facility_name())()
+        # Check configuration of facility and pass names of missing settings to context as 'missing_configurations'.
+        try:
+            context['missing_configurations'] = ", ".join(facility.facility_settings.get_unconfigured_settings())
+        except AttributeError:
+            context['missing_configurations'] = ''
+        return context
+
     def get_form_class(self):
         facility_name = self.get_facility_name()
 
@@ -562,7 +650,9 @@ class ObservationTemplateCreateView(FormView):
             raise ValueError('Must provide a facility name')
 
         # TODO: modify this to work with all LCO forms
-        return get_service_class(facility_name)().get_template_form(None)
+        facility = get_service_class(facility_name)()
+        facility.set_user(self.request.user)
+        return facility.get_template_form(None)
 
     def get_form(self, form_class=None):
         form = super().get_form()
@@ -592,9 +682,11 @@ class ObservationTemplateUpdateView(LoginRequiredMixin, FormView):
 
     def get_form_class(self):
         self.object = self.get_object()
-        return get_service_class(self.object.facility)().get_template_form(None)
+        facility = get_service_class(self.object.facility)()
+        facility.set_user(self.request.user)
+        return facility.get_template_form(None)
 
-    def get_form(self):
+    def get_form(self, form_class=None):
         form = super().get_form()
         form.helper.form_action = reverse(
             'tom_observations:template-update', kwargs={'pk': self.object.id}
@@ -603,7 +695,7 @@ class ObservationTemplateUpdateView(LoginRequiredMixin, FormView):
 
     def get_initial(self):
         initial = super().get_initial()
-        initial.update(self.object.parameters_as_dict)
+        initial.update(self.object.parameters)
         initial['facility'] = self.object.facility
         return initial
 
@@ -618,3 +710,7 @@ class ObservationTemplateDeleteView(LoginRequiredMixin, DeleteView):
     """
     model = ObservationTemplate
     success_url = reverse_lazy('tom_observations:template-list')
+
+
+class FacilityStatusView(TemplateView):
+    template_name = 'tom_observations/facility_status.html'
